/*
 * Including copyright notice and other comments from pre-ONTAP 9.7 version
 * of crc32.c.
 *
 * crc32.c -- compute the CRC-32 of a data stream
 * Copyright (C) 1995-2002 Mark Adler
 * For conditions of distribution and use, see copyright notice in zlib.h
 *
 * (From pre-ONTAP 9.7 version of crc32.c comment block before make_crc_table()
 *
 *  Generate a table for a byte-wise 32-bit CRC calculation on the polynomial:
 *  x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x+1.
 *
 *  Polynomials over GF(2) are represented in binary, one bit per coefficient,
 *  with the lowest powers in the most significant bit.  Then adding polynomials
 *  is just exclusive-or, and multiplying a polynomial by x is a right shift by
 *  one.  If we call the above polynomial p, and represent a byte as the
 *  polynomial q, also with the lowest power in the most significant bit (so the
 *  byte 0xb1 is the polynomial x^7+x^3+x+1), then the CRC is (q*x^32) mod p,
 *  where a mod b means the remainder after dividing a by b.
 *
 *  This calculation is done using the shift-register method of multiplying and
 *  taking the remainder.  The register is initialized to zero, and for each
 *  incoming bit, x^32 is added mod p to the register if the bit is a one (where
 *  x^32 mod p is p+x^32 = x^26+...+1), and the register is multiplied mod p by
 *  x (which is shifting right by one and adding x^32 mod p if the bit shifted
 *  out is a one).  We start with the highest power (least significant bit) of
 *  q and repeat for all eight bits of q.
 *
 *  The table is simply the CRC of all possible eight bit values.  This is all
 *  the information needed to generate CRC's on data a byte at a time for all
 *  combinations of CRC register values and incoming bytes.
 */

#include <zlib/zlib.h>
#include <libisal_raid/crc.h>

/* ========================================================================= */
uLong zlib_crc32(uLong crc, const Bytef *buf, uInt len)
{
   /*
    * The polynomial corresponding to the above is 0x04C11DB7, this corresponds
    * to the CRC32 IEEE standard as opposed to 0xEDB88320 which is used for
    * the reflected CRC32 IEEE (crc32_gzip_refl) version.
    */
    return crc32_gzip_refl_base(crc, (uint8_t*)buf, len);
}
